import { Component } from '@tarojs/taro';
import { ComponentClass, ReactElement } from 'react';
import { connect } from '@tarojs/redux';
import { UserState, updateUserInfo } from '../store/rootReducers/userinfo';
import {
  DeviceState,
  updateDeviceData,
  resetDeviceData,
} from '../store/rootReducers/device';
import PrepayConfig from '../types/Order/Order';
import { updatePayData } from '../store/rootReducers/prepayConfig';
import { Customer } from '../types/Customer/Customer';
import Taro from '@tarojs/taro';
import { registerAndLogin } from '../api/customer';
import { getDeviceConfig } from '../api/device';

type DeviceEntity = {
  serviceId: number;
  equipmentNum: string;
};

export type LoginAndScanExportProps = {
  loginHandle: () => Promise<void>;
  scanHandle: () => void;
  getDeviceEntity: (paramStr: string) => DeviceEntity;
  getDeviceConfigHandle: ({
    equipmentNum,
    serviceId,
  }: DeviceEntity) => Promise<void>;
};

type PageStateProps = {
  userinfo: Customer;
};

type PageDispatchProps = {
  updateUserInfo: (e: UserState) => void;
  updateDeviceData: (e: DeviceState) => void;
  updatePayData: (e: PrepayConfig[]) => void;
  resetDeviceData: () => void;
};

type PageOwnProps = {
  render: (e: LoginAndScanExportProps) => ReactElement;
};

interface LoginAndScan {
  props: PageStateProps & PageDispatchProps & PageOwnProps;
}

@connect(
  ({ userinfo }) => ({
    userinfo,
  }),
  dispatch => ({
    updateUserInfo(data: UserState) {
      dispatch(updateUserInfo(data));
    },
    updateDeviceData(data: DeviceState) {
      dispatch(updateDeviceData(data));
    },
    updatePayData(data: PrepayConfig[]) {
      dispatch(updatePayData(data));
    },
    resetDeviceData() {
      dispatch(resetDeviceData());
    },
  }),
)
class LoginAndScan extends Component {
  loginHandle() {
    const { updateUserInfo } = this.props;

    return Taro.login().then(res => {
      const { code } = res;
      return registerAndLogin({
        code,
        // userName: '',
      }).then(res => {
        const { token, customerId } = res;
        updateUserInfo({
          token,
          customerId,
        });
      });
    });
  }

  scanHandle() {
    Taro.scanCode({
      onlyFromCamera: true,
      scanType: ['qrCode'],
    })
      .then(res => {
        console.log(res);
        const { path, result } = res;
        if (path) {
          let queryArr = path.split('?');
          let queryStr = queryArr.length >= 2 ? queryArr[1] : '';
          console.log(result, queryArr, queryStr);
          const entity = this.getDeviceEntity(queryStr);
          this.getDeviceConfigHandle(entity);
        } else {
          Taro.showToast({
            title: '请扫描正确的小程序',
            icon: 'none',
          });
        }
      })
      .catch(err => {
        console.error(err);
      });
  }

  getDeviceConfigHandle({ equipmentNum, serviceId }: DeviceEntity) {
    Taro.showLoading();
    const {
      userinfo,
      updateDeviceData,
      updatePayData,
      resetDeviceData,
    } = this.props;
    return getDeviceConfig({
      customerId: userinfo.customerId,
      equipmentNum: equipmentNum,
      serviceId: serviceId,
    })
      .then(res => {
        Taro.hideLoading();
        console.log(res);
        const { deviceInfoResponse, prepayConfigs } = res;
        if (deviceInfoResponse.isUsed) {
          console.log('设备使用中');
          return;
        }
        if (deviceInfoResponse) {
          updateDeviceData(deviceInfoResponse);
        } else {
          resetDeviceData();
        }
        if (prepayConfigs) {
          updatePayData(prepayConfigs);
        } else {
          updatePayData([]);
        }
        Taro.navigateTo({
          url: '/pages/pay/pay',
        });
      })
      .catch(err => {
        console.error(err);
        Taro.showToast({
          title: err.msg || '请扫描正确的设备码',
          icon: 'none',
        });
      });
  }

  getDeviceEntity(paramStr: string): DeviceEntity {
    let serviceId = Number(paramStr.slice(0, 2));
    let equipmentNum = paramStr.slice(2);
    return {
      serviceId,
      equipmentNum,
    };
  }

  render() {
    return this.props.render({
      loginHandle: this.loginHandle,
      scanHandle: this.scanHandle,
      getDeviceEntity: this.getDeviceEntity,
      getDeviceConfigHandle: this.getDeviceConfigHandle,
    });
  }
}

export default LoginAndScan;
